using System;
using System.Collections;
using System.Linq;
using UnityEngine;
using Random = UnityEngine.Random;

namespace CheesyFX
{
    public class Throat : Orifice
    {
        private float nippleStim;
        private static IEnumerator blinkRestore;

        public override void Init(string name)
        {
            relaxation = new JSONStorableFloat("Relaxation", 0f, 0f, 1f);
            base.Init(name);
            DAZCharacterSelector characterSelector = FillMeUp.atom.GetStorableByID("geometry") as DAZCharacterSelector;
            depthReference = FillMeUp.atom.rigidbodies.First(x => x.name == "ThroatTrigger");
            name = "Throat";
            // onEnterActions = orificeManager.bodyManager.audioManager.Stop;
            sensitivity.val = sensitivity.defaultVal = .2f;
            audioSource = depthReference.gameObject.AddComponent<AudioSource>();

            bulgeScale.val = bulgeScale.defaultVal = 2f;
            bulgeDepthScale.val = bulgeDepthScale.defaultVal = 1.1f;
            
            thrustForce.amplitude.mean.SetWithDefault(150f);
            thrustForce.amplitude.delta.SetWithDefault(50f);
            thrustForce.period.mean.SetWithDefault(.9f);
            thrustForce.period.delta.SetWithDefault(.4f);
            
            maleForce.amplitude.mean.SetWithDefault(150f);
            maleForce.amplitude.delta.SetWithDefault(50f);

            torque = gameObject.AddComponent<Torque>().Init(name+":Torque", rb, () => -rb.transform.right);
            torque.AddSync(thrustForce);
            torque.sync.enabled = true;
            torque.amplitude.mean.SetWithDefault(15f);
            torque.amplitude.delta.SetWithDefault(5f);
            torque.amplitude.sharpness.SetWithDefault(2f);
            torque.amplitude.transitionQuicknessMean.SetWithDefault(2f);
            torque.sync.phaseOffsetMean.SetWithDefault(0f);
            torque.sync.phaseOffsetDelta.SetWithDefault(.2f);
            torque1 = gameObject.AddComponent<Torque>().Init(name+":Torque1", rb, () => -rb.transform.forward);
            torque1.constant.val = true;
            torque1.amplitude.mean.SetWithDefault(0f);
            torque1.amplitude.delta.SetWithDefault(30f);
            torque1.amplitude.sharpness.SetWithDefault(2f);
            // var head = FillMeUp.atom.rigidbodies.First(x => x.name == "head");
            // strokeForce = gameObject.AddComponent<Force>().Init("StrokeForce", head, () => -forward, 100f);
            // strokeForce.amplitude.mean.val = 150f;
            // strokeForce.amplitude.delta.val = 100f;
            // strokeForce.period.mean.val = .6f;
            // strokeForce.period.sharpness.val = 3f;
        }

        public override void SetPenetrated(bool val)
        {
            base.SetPenetrated(val);
            if (val)
            {
                zeroRelaxation.Stop();
                Gagger.RestoreBlink();
                BJHelper.singleton.enabled = false;
                FillMeUp.eyelidBehavior.blinkSpaceMax = 10f;
                FillMeUp.eyelidBehavior.blinkSpaceMin = 2f;
                FillMeUp.eyelidBehavior.blinkTimeMin = .75f;
                FillMeUp.eyelidBehavior.blinkTimeMax = 2f;
                FillMeUp.eyelidBehavior.blinkDownUpRatio = .8f;
                if (Random.Range(0f, 1f) > .25f) FillMeUp.eyelidBehavior.Blink();
                blinkRestore.Stop();
                blinkRestore = BlinkRestore().Start();
            }
            else
            {
                // Gagger.RestoreBlink();
                if(BJHelper.penetrators.Count > 0) BJHelper.singleton.enabled = true;
                zeroStretch = ZeroStretch().Start();
                zeroRelaxation = ZeroRelaxation().Start();
                enabled = false;
            }
        }

        private static IEnumerator BlinkRestore()
        {
            yield return new WaitForSeconds(4f);
            Gagger.RestoreBlink();
        }

        public override void FixedUpdate()
        {
            base.FixedUpdate();
            var absSpeed = Mathf.Abs(speed.val);
            if(!audioSource.isPlaying && penetrationSoundsVolume.val > 0 && absSpeed > .05f && depth.val > .05f)
            {
                var clip = FillMeUp.bjLibrary.GetRandomClip();
                audioSource.clip = clip;
                audioSource.volume = 10f * absSpeed * (1f + depth.val)*(1f + depth.val) * penetrationSoundsVolume.val;
                audioSource.Play();
            }

            if (depth.val > .1f)
            {
                nippleStim = (depth.val - .1f) * Time.fixedDeltaTime * 6f;
                NippleManager.nippleDrivers[0].Stimulate(nippleStim);
                NippleManager.nippleDrivers[1].Stimulate(nippleStim);
                if (depth.val > Gagger.gagThreshold.val) relaxation.val += .0005f;
            }

            // if (depth.val > Gagger.gagThreshold.val && Random.Range(0f, 1f) > relaxation.val)
            // {
            //     if(Gagger.gag == null && lastDepth <= Gagger.gagThreshold.val) Gagger.gag = Gagger.Gag().Start();
            // }
            // if (Gagger.gag != null && depth.val < Gagger.gagThreshold.val - .02f)
            // {
            //     Gagger.Stop();
            // }
            
            Gagger.Update(depth.val, lastDepth, relaxation);

            if (relaxation.val > 0f) relaxation.val -= .0125f * Time.fixedDeltaTime;
            else
            {
                relaxation.val = 0f;
            }
            lastDepth = depth.val;
            // forward = -penetrator.forward();
        }

        private WaitForFixedUpdate wait = new WaitForFixedUpdate();
        private IEnumerator zeroRelaxation;
        private IEnumerator ZeroRelaxation()
        {
            while (relaxation.val > 0f)
            {
                relaxation.val -= .075f * Time.fixedDeltaTime;
                yield return wait;
            }
            relaxation.val = 0f;
        }

        public override float Bulge(int x)
        {
            float val = (3f*penetratorWidth)*bulgeScale.val*Mathf.Min(10f * depth.val * bulgeDepthScale.val, 1f) * Mathf.Pow(2f, -x * x *1e-4f* Mathf.Pow(depth.val*bulgeDepthScale.val, -4));
            return val;
        }

        public override void CreateSettingsUI()
        {
            base.CreateSettingsUI();
            Gagger.gagThreshold.CreateUI(UIElements);
            Gagger.enabled.CreateUI(UIElements, true);
            Gagger.gagScale.CreateUI(UIElements, true);
        }
        
        public override void CreateBulgeUI()
        {
            bulgeScale.CreateUI(FillMeUp.singleton, UIElements:UIElements);
            bulgeDepthScale.CreateUI(FillMeUp.singleton, UIElements:UIElements);
        }

        public override void RegisterCollision(Collider collider)
        {
            penetrator = GetPenetrator(collider);
            forceTarget = penetrator.rootTransform;
            depthMeter.transform.parent = penetrator.tip;
            depthMeter.transform.localPosition = Vector3.zero;
            penetratorWidth = penetrator.width * stretchScale.val;
            isPenetrated = true;
            penetratingAtom = penetrator.atom;
        }

        public override void OnDestroy()
        {
            base.OnDestroy();
            Gagger.gag.Stop();
            zeroRelaxation.Stop();
        }
        
        public void OnDisable()
        {
            Gagger.gag.Stop();
            forward = Vector3.zero;
            // thrustForce.enabled = false;
        }
    }
}