using System.Collections.Generic;
using UnityEngine;
using Object = UnityEngine.Object;

namespace CheesyFX
{
    public static class Debug
    {
        
        private static List<DebugObject> debugObjects = new List<DebugObject>();
        public static void Draw(this Transform t)
        {
            debugObjects.Add(new DebugLine(t,0));
            debugObjects.Add(new DebugLine(t,1));
            debugObjects.Add(new DebugLine(t,2));
        }

        public static void Clear()
        {
            debugObjects.ForEach(x => x.Destroy());
            debugObjects.Clear();
        }
    }

    public class DebugObject
    {
        private GameObject go;
        
        public void Destroy()
        {
            Object.Destroy(go);
        }
    }

    public class DebugLine : DebugObject
    {
        private static Shader debugShader = Shader.Find("Battlehub/RTGizmos/Handles");
        private Material material = new Material(Shader.Find("Battlehub/RTGizmos/Handles"));
        
        private LineRenderer lineRenderer;

        private Transform t;
        private GameObject go;
        private Vector3 direction;
        private float length;

        public DebugLine(Transform t, Vector3 direction)
        {
            this.t = t;
            this.direction = direction;
            length = .1f / Vector3.Dot(t.lossyScale, direction);
            go = new GameObject();
            go.transform.SetParent(t, false);
            lineRenderer = go.AddComponent<LineRenderer>();
            lineRenderer.material = material;
            lineRenderer.widthMultiplier = 0.003f;
            lineRenderer.positionCount = 2;
            lineRenderer.useWorldSpace = false;
            // lineRenderer.SetPositions(new []{t.position, t.position + t.up*.1f});
            lineRenderer.SetPositions(new []{Vector3.zero, this.direction*length});
            material.color = Color.cyan;
        }
        
        public DebugLine(Transform t, int axis)
        {
            this.t = t;
            switch (axis)
            {
                case 0:
                {
                    direction = Vector3.right;
                    material.color = Color.red;
                    break;
                }
                case 1:
                {
                    direction = Vector3.up;
                    material.color = Color.green;
                    break;
                }
                case 2:
                {
                    direction = Vector3.forward;
                    material.color = Color.blue;
                    break;
                }
            }
            length = .1f / t.lossyScale[axis];
            go = new GameObject();
            go.transform.SetParent(t, false);
            lineRenderer = go.AddComponent<LineRenderer>();
            lineRenderer.material = material;
            lineRenderer.widthMultiplier = 0.003f;
            lineRenderer.positionCount = 2;
            lineRenderer.useWorldSpace = false;
            // lineRenderer.SetPositions(new []{t.position, t.position + t.up*.1f});
            lineRenderer.SetPositions(new []{Vector3.zero, this.direction*length});
        }
    }
}